/*
 * Copyright (C) 2001,2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#ifndef _CAPSEL_SYS_H
#define _CAPSEL_SYS_H

#include <linux/config.h>
#include <linux/types.h>
#include <linux/capability.h>
#include <linux/unistd.h>
#include <linux/sched.h>

#include "config.h"
#include "rule.h"

typedef struct {
	int number;
	void ** old;
	void * new;
} hook_t;

extern hook_t sys_hooks[];

void install_hook(hook_t *);
void uninstall_hook(hook_t *);
int clear_environ(void);

static inline void new_cap_emulate_setxuid(const rule_t * rule,
	kernel_cap_t effective, kernel_cap_t permitted,
	uid_t old_ruid, uid_t old_euid, uid_t old_suid)
{
	uid_t uid;

#ifdef KEEP_CAPABILITIES
	/*
	 * If process uses via prctl() new keep_capabilities feature 
	 * (introduced in 2.2.18), we do not override original 
	 * cap_emulate_setxuid and uidcap option has no effect.
	 */
	if (current->keep_capabilities)	return;
#endif

	/*
	 * We do not change the behavior of setuid if default value is
	 * used in uidcap option.
	 */
	if (!memcmp(&rule->uidcaps, &UIDCAP_DEFAULT, sizeof(capabilities_t)))
		return;
	
    	uid=rule->uid;

	if ((old_ruid==uid || old_euid==uid || old_suid==uid) &&
	    (current->uid!=uid && current->euid!=uid && current->suid!=uid)) {
		/* instead of: cap_clear(current->cap_permitted); */
		permitted=cap_combine(permitted, rule->uidcaps.permitted.raise);
		permitted=cap_drop(permitted, rule->uidcaps.permitted.lower);
		/* instead of: cap_clear(current->cap_effective); */
	    	effective=cap_combine(effective, rule->uidcaps.effective.raise);
		effective=cap_drop(effective, rule->uidcaps.effective.lower);
	}
	
	if (old_euid==uid && current->euid!=uid) {
		/* instead of: cap_clear(current->cap_effective); */
	    	effective=cap_combine(effective, rule->uidcaps.effective.raise);
		effective=cap_drop(effective, rule->uidcaps.effective.lower);
	}
	
	if (old_euid!=uid && current->euid==uid)
		effective=permitted;
	
	current->cap_effective=effective;
	current->cap_permitted=permitted;
} /* new_cap_emulate_setxuid */

#endif /* _CAPSEL_SYS_H */
